@@ -1,6 +1,6 @@
-// Copyright (C) 2011-2012 Ryan Curtin
+// Copyright (C) 2011-2014 Ryan Curtin
+// Copyright (C) 2012-2014 Conrad Sanderson
 // Copyright (C) 2011 Matthew Amidon
-// Copyright (C) 2012 Conrad Sanderson
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,10 +13,23 @@
 // SpMat::iterator_base implementation                                       //
 ///////////////////////////////////////////////////////////////////////////////
 
+
+template<typename eT>
+inline
+SpMat<eT>::iterator_base::iterator_base()
+  : M(NULL)
+  , internal_col(0)
+  , internal_pos(0)
+  {
+  // Technically this iterator is invalid (it may not point to a real element)
+  }
+
+
+
 template<typename eT>
 inline
 SpMat<eT>::iterator_base::iterator_base(const SpMat<eT>& in_M)
-  : M(in_M)
+  : M(&in_M)
   , internal_col(0)
   , internal_pos(0)
   {
@@ -28,7 +41,7 @@ SpMat<eT>::iterator_base::iterator_base(const SpMat<eT>& in_M)
 template<typename eT>
 inline
 SpMat<eT>::iterator_base::iterator_base(const SpMat<eT>& in_M, const uword in_col, const uword in_pos)
-  : M(in_M)
+  : M(&in_M)
   , internal_col(in_col)
   , internal_pos(in_pos)
   {
@@ -43,7 +56,7 @@ arma_hot
 eT
 SpMat<eT>::iterator_base::operator*() const
   {
-  return M.values[internal_pos];
+  return M->values[internal_pos];
   }
 
 
@@ -52,6 +65,13 @@ SpMat<eT>::iterator_base::operator*() const
 // SpMat::const_iterator implementation                                      //
 ///////////////////////////////////////////////////////////////////////////////
 
+template<typename eT>
+inline
+SpMat<eT>::const_iterator::const_iterator()
+  : iterator_base()
+  {
+  }
+
 template<typename eT>
 inline
 SpMat<eT>::const_iterator::const_iterator(const SpMat<eT>& in_M, uword initial_pos)
@@ -65,7 +85,7 @@ SpMat<eT>::const_iterator::const_iterator(const SpMat<eT>& in_M, uword initial_p
     }
 
   // Determine which column we should be in.
-  while(iterator_base::M.col_ptrs[iterator_base::internal_col + 1] <= iterator_base::internal_pos)
+  while(iterator_base::M->col_ptrs[iterator_base::internal_col + 1] <= iterator_base::internal_pos)
     {
     iterator_base::internal_col++;
     }
@@ -79,16 +99,16 @@ SpMat<eT>::const_iterator::const_iterator(const SpMat<eT>& in_M, uword in_row, u
   : iterator_base(in_M, in_col, 0)
   {
   // So we have a position we want to be right after.  Skip to the column.
-  iterator_base::internal_pos = iterator_base::M.col_ptrs[iterator_base::internal_col];
+  iterator_base::internal_pos = iterator_base::M->col_ptrs[iterator_base::internal_col];
 
   // Now we have to make sure that is the right column.
-  while(iterator_base::M.col_ptrs[iterator_base::internal_col + 1] <= iterator_base::internal_pos)
+  while(iterator_base::M->col_ptrs[iterator_base::internal_col + 1] <= iterator_base::internal_pos)
     {
     iterator_base::internal_col++;
     }
 
   // Now we have to get to the right row.
-  while((iterator_base::M.row_indices[iterator_base::internal_pos] < in_row) && (iterator_base::internal_col == in_col))
+  while((iterator_base::M->row_indices[iterator_base::internal_pos] < in_row) && (iterator_base::internal_col == in_col))
     {
     ++(*this); // Increment iterator.
     }
@@ -109,7 +129,7 @@ SpMat<eT>::const_iterator::const_iterator(const SpMat<eT>& in_M, const uword /*
 template<typename eT>
 inline
 SpMat<eT>::const_iterator::const_iterator(const typename SpMat<eT>::const_iterator& other)
-  : iterator_base(other.M, other.internal_col, other.internal_pos)
+  : iterator_base(*other.M, other.internal_col, other.internal_pos)
   {
   // Nothing to do.
   }
@@ -124,14 +144,14 @@ SpMat<eT>::const_iterator::operator++()
   {
   ++iterator_base::internal_pos;
 
-  if (iterator_base::internal_pos == iterator_base::M.n_nonzero)
+  if (iterator_base::internal_pos == iterator_base::M->n_nonzero)
     {
-    iterator_base::internal_col = iterator_base::M.n_cols;
+    iterator_base::internal_col = iterator_base::M->n_cols;
     return *this;
     }
 
   // Check to see if we moved a column.
-  while (iterator_base::M.col_ptrs[iterator_base::internal_col + 1] <= iterator_base::internal_pos)
+  while (iterator_base::M->col_ptrs[iterator_base::internal_col + 1] <= iterator_base::internal_pos)
     {
     ++iterator_base::internal_col;
     }
@@ -171,7 +191,7 @@ SpMat<eT>::const_iterator::operator--()
   // printf("now pos %d\n", iterator_base::internal_pos);
 
   // First, see if we moved back a column.
-  while (iterator_base::internal_pos < iterator_base::M.col_ptrs[iterator_base::internal_col])
+  while (iterator_base::internal_pos < iterator_base::M->col_ptrs[iterator_base::internal_col])
     {
     // printf("colptr %d (col %d)\n", iterator_base::M.col_ptrs[iterator_base::internal_col], iterator_base::internal_col);
     
@@ -298,10 +318,10 @@ SpValProxy<SpMat<eT> >
 SpMat<eT>::iterator::operator*()
   {
   return SpValProxy<SpMat<eT> >(
-    iterator_base::M.row_indices[iterator_base::internal_pos],
+    iterator_base::M->row_indices[iterator_base::internal_pos],
     iterator_base::internal_col,
-    access::rw(iterator_base::M),
-    &access::rw(iterator_base::M.values[iterator_base::internal_pos]));
+    access::rw(*iterator_base::M),
+    &access::rw(iterator_base::M->values[iterator_base::internal_pos]));
   }
 
 
@@ -367,6 +387,18 @@ SpMat<eT>::iterator::operator--(int)
 /**
  * Initialize the const_row_iterator.
  */
+
+template<typename eT>
+inline
+SpMat<eT>::const_row_iterator::const_row_iterator()
+  : iterator_base()
+  , internal_row(0)
+  , actual_pos(0)
+  {
+  }
+
+
+
 template<typename eT>
 inline
 SpMat<eT>::const_row_iterator::const_row_iterator(const SpMat<eT>& in_M, uword initial_pos)
@@ -390,23 +422,23 @@ SpMat<eT>::const_row_iterator::const_row_iterator(const SpMat<eT>& in_M, uword i
   // row.  What we will do is loop across all columns looking for elements in
   // row 0 (and add to our sum), then in row 1, and so forth, until we get to
   // the desired position.
-  uword cur_pos = -1;
+  uword cur_pos = std::numeric_limits<uword>::max(); // Invalid value.
   uword cur_row = 0;
   uword cur_col = 0;
 
   while(true) // This loop is terminated from the inside.
     {
     // Is there anything in the column we are looking at?
-    for (uword ind = 0; ((iterator_base::M.col_ptrs[cur_col] + ind < iterator_base::M.col_ptrs[cur_col + 1]) && (iterator_base::M.row_indices[iterator_base::M.col_ptrs[cur_col] + ind] <= cur_row)); ind++)
+    for (uword ind = 0; ((iterator_base::M->col_ptrs[cur_col] + ind < iterator_base::M->col_ptrs[cur_col + 1]) && (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] <= cur_row)); ind++)
       {
       // There is something in this column.  Is it in the row we are looking at?
-      const uword row_index = iterator_base::M.row_indices[iterator_base::M.col_ptrs[cur_col] + ind];
+      const uword row_index = iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind];
       if (row_index == cur_row)
         {
         // Yes, it is what we are looking for.  Increment our current position.
         if (++cur_pos == iterator_base::internal_pos)
           {
-          actual_pos = iterator_base::M.col_ptrs[cur_col] + ind;
+          actual_pos = iterator_base::M->col_ptrs[cur_col] + ind;
           internal_row = cur_row;
           iterator_base::internal_col = cur_col;
 
@@ -423,7 +455,7 @@ SpMat<eT>::const_row_iterator::const_row_iterator(const SpMat<eT>& in_M, uword i
       }
 
     cur_col++; // Done with the column.  Move on.
-    if (cur_col == iterator_base::M.n_cols)
+    if (cur_col == iterator_base::M->n_cols)
       {
       // We are out of columns.  Loop back to the beginning and look on the
       // next row.
@@ -465,7 +497,7 @@ SpMat<eT>::const_row_iterator::const_row_iterator(const SpMat<eT>& in_M, uword i
 template<typename eT>
 inline
 SpMat<eT>::const_row_iterator::const_row_iterator(const typename SpMat<eT>::const_row_iterator& other)
-  : iterator_base(other.M, other.internal_col, other.internal_pos)
+  : iterator_base(*other.M, other.internal_col, other.internal_pos)
   , internal_row(other.internal_row)
   , actual_pos(other.actual_pos)
   {
@@ -486,11 +518,11 @@ SpMat<eT>::const_row_iterator::operator++()
   // We just need to find the next nonzero element.
   iterator_base::internal_pos++;
 
-  if(iterator_base::internal_pos == iterator_base::M.n_nonzero)
+  if(iterator_base::internal_pos == iterator_base::M->n_nonzero)
     {
-    internal_row = iterator_base::M.n_rows;
+    internal_row = iterator_base::M->n_rows;
     iterator_base::internal_col = 0;
-    actual_pos = iterator_base::M.n_nonzero;
+    actual_pos = iterator_base::M->n_nonzero;
 
     return *this;
     }
@@ -502,21 +534,21 @@ SpMat<eT>::const_row_iterator::operator++()
   while (true) // This loop is terminated from the inside.
     {
     // Increment the current column and see if we are now on a new row.
-    if (++cur_col == iterator_base::M.n_cols)
+    if (++cur_col == iterator_base::M->n_cols)
       {
       cur_col = 0;
       cur_row++;
       }
 
     // Is there anything in this new column?
-    for (uword ind = 0; ((iterator_base::M.col_ptrs[cur_col] + ind < iterator_base::M.col_ptrs[cur_col + 1]) && (iterator_base::M.row_indices[iterator_base::M.col_ptrs[cur_col] + ind] <= cur_row)); ind++)
+    for (uword ind = 0; ((iterator_base::M->col_ptrs[cur_col] + ind < iterator_base::M->col_ptrs[cur_col + 1]) && (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] <= cur_row)); ind++)
       {
-      if (iterator_base::M.row_indices[iterator_base::M.col_ptrs[cur_col] + ind] == cur_row)
+      if (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] == cur_row)
         {
         // We have successfully incremented.
         internal_row = cur_row;
         iterator_base::internal_col = cur_col;
-        actual_pos = iterator_base::M.col_ptrs[cur_col] + ind;
+        actual_pos = iterator_base::M->col_ptrs[cur_col] + ind;
 
         return *this; // Now we are done.
         }
@@ -562,21 +594,21 @@ SpMat<eT>::const_row_iterator::operator--()
   while (true) // This loop is terminated from the inside.
     {
     // Decrement the current column and see if we are now on a new row.  This is a uword so a negativity check won't work.
-    if (--cur_col > iterator_base::M.n_cols /* this means it underflew */)
+    if (--cur_col > iterator_base::M->n_cols /* this means it underflew */)
       {
-      cur_col = iterator_base::M.n_cols - 1;
+      cur_col = iterator_base::M->n_cols - 1;
       cur_row--;
       }
 
     // Is there anything in this new column?
-    for (uword ind = 0; ((iterator_base::M.col_ptrs[cur_col] + ind < iterator_base::M.col_ptrs[cur_col + 1]) && (iterator_base::M.row_indices[iterator_base::M.col_ptrs[cur_col] + ind] <= cur_row)); ind++)
+    for (uword ind = 0; ((iterator_base::M->col_ptrs[cur_col] + ind < iterator_base::M->col_ptrs[cur_col + 1]) && (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] <= cur_row)); ind++)
       {
-      if (iterator_base::M.row_indices[iterator_base::M.col_ptrs[cur_col] + ind] == cur_row)
+      if (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] == cur_row)
         {
         // We have successfully decremented.
         iterator_base::internal_col = cur_col;
         internal_row = cur_row;
-        actual_pos = iterator_base::M.col_ptrs[cur_col] + ind;
+        actual_pos = iterator_base::M->col_ptrs[cur_col] + ind;
 
         return *this; // Now we are done.
         }
@@ -705,8 +737,8 @@ SpMat<eT>::row_iterator::operator*()
   return SpValProxy<SpMat<eT> >(
     const_row_iterator::internal_row,
     iterator_base::internal_col,
-    access::rw(iterator_base::M),
-    &access::rw(iterator_base::M.values[const_row_iterator::actual_pos]));
+    access::rw(*iterator_base::M),
+    &access::rw(iterator_base::M->values[const_row_iterator::actual_pos]));
   }
 
 
